--[[
DK Sounds (Plays Sounds for some DK's spells and Detect some Procs)
Created by NTex

Changes Log:
NTEX 2008/11/30: Created baseline code
]]

-- Create the base variable array
DKSounds = {}

-- General stuff
DKSounds.Version = "0.8b9";
DKSounds.Title = "DK Sounds " .. DKSounds.Version;
DKSounds.Debug = 0

-- So we can search later on, for
-- our casts and don't grab all the
-- others spells from the others DKs
DKSounds.CharName = UnitName("player");

-- Events Table
DKSounds.events = {}

-- Invisible frame to grab events
DKSounds.eventFrame = CreateFrame("Frame")
DKSounds.eventFrame:SetScript("OnEvent",
    function(self, event, ...)
        DKSounds.events[event](...)
    end)

DKSounds.eventFrame:RegisterEvent("ADDON_LOADED")
--DKSounds.eventFrame:RegisterEvent("VARIABLES_LOADED")
DKSounds.eventFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
-- For detecting Winged Steed of the Ebon Blade
-- since it's physical school, also perhaps we can
-- change Acherus Deathcharger
DKSounds.eventFrame:RegisterEvent("UNIT_AURA")

-- Store names on variables so
-- we can search for them later
-- Also if we do this like I hope
-- to avoid localizations issues
DKSounds.Spells = {}
DKSounds.Spells.DG = GetSpellInfo(49576)
DKSounds.Spells.RS = GetSpellInfo(56815)
DKSounds.Spells.Str = GetSpellInfo(47476)
DKSounds.Spells.RD = GetSpellInfo(52150)
DKSounds.Spells.HW = GetSpellInfo(57330)
DKSounds.Spells.BB = GetSpellInfo(48721)
DKSounds.Spells.BS = GetSpellInfo(45902)
DKSounds.Spells.SS = GetSpellInfo(55090)
DKSounds.Spells.Obl = GetSpellInfo(49020)
DKSounds.Spells.Pes = GetSpellInfo(50842)
DKSounds.Spells.UB = GetSpellInfo(49194)
DKSounds.Spells.SG = GetSpellInfo(49206)
DKSounds.Spells.AD = GetSpellInfo(48778)
DKSounds.Spells.WS = GetSpellInfo(54729)
DKSounds.Spells.Cann = GetSpellInfo(20577)
DKSounds.Spells.AoD = GetSpellInfo(42650)
DKSounds.Spells.FS = GetSpellInfo(49143)
DKSounds.Spells.HS = GetSpellInfo(55050)

-- Global to avoid spamming msg/sound everytime UNIT_AURA fires
DKSounds.Spells.Mounted = IsMounted(DKSounds.CharName)

-- Spellnames: ID's
-- Deathgrip: 49576
-- Rune Strike: 56815
-- Strangulate: 47476
-- Raise Dead: 52150 (old - 46584)
-- Horn of Winter: 57330
-- Blood Boil: 48721
-- Blood Strike: 45902 
-- Scourge Strike: 55090
-- Obliterate: 49020
-- Pestilence: 50842
-- Unholy Blight: 49194
-- Summon Gargoyle:     
-- Acherus Deathcharger: 48778
-- Winged Steed of the Ebon Blade: 54729
-- Cannibalize: 20577
-- Army of the Dead: 42650
-- Frost Strike: 49143
-- Heart Strike: 55050

-- Variables to not spam Rune Strike emote
DKSounds.Spells.RSCounter = 0
DKSounds.Spells.RSEnabled = false 

function DKSounds.events.ADDON_LOADED(addon)
    if addon ~= "DK_Sounds" then
        return
    end

    local _ ,EngClass = UnitClass("player")
   
    -- If class isn't DK unregister
    -- everything to free up resources
    if (EngClass ~= "DEATHKNIGHT") then
        --DKSounds.eventFrame:UnregisterAllEvents()
        DKSounds.eventFrame:UnregisterEvent("ADDON_LOADED")
        DKSounds.eventFrame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
        DKSounds.eventFrame:UnregisterEvent("UNIT_AURA")
        return 
    end

    -- Create Default saved variables
    -- if it doesn't exist yet
    if (not DKSoundsData) then 
        DKSoundsData = { };
        DKSoundsData.config = { };
        -- In the future with this we can
        -- create convert functions without
        -- forcing the reset of the whole
        -- Configuration
        DKSoundsData.config.cfgver = 0;
        -- Settings for All Options
        DKSoundsData.config.Msgs = true;
        DKSoundsData.config.Snds = true;
        -- Individual options settings (sounds)
        DKSoundsData.config.DG = true;
        DKSoundsData.config.RS = true;
        DKSoundsData.config.Str = true;
        DKSoundsData.config.RD = true;
        DKSoundsData.config.HW = true;
        DKSoundsData.config.BB = true;
        DKSoundsData.config.BS = true;
        DKSoundsData.config.SS = true;
        DKSoundsData.config.Obl = true;
        DKSoundsData.config.Pes = true;
        DKSoundsData.config.UB = true;
        DKSoundsData.config.SG = true;
        DKSoundsData.config.Mount = true;
        DKSoundsData.config.Cann = true;
        DKSoundsData.config.AoD = true;
    else
-- Future additions: (when we gather a lot change the cfg file version)
        -- New addition
        if (DKSoundsData.config.AoD == nil) then
            DKSoundsData.config.AoD = true;
        end
-- Future additions: (when we gather a lot change the cfg file version)
    end    

    -- Interface - Options
    DKSounds:CreateOptionFrame()
    
    -- Slash Options
    SlashCmdList["DKSounds"] = DKSounds.Options
        SLASH_DKSounds1 = "/DKSounds"
        SLASH_DKSounds2 = "/DKS"

    local color0 = "|c00F76541"
    local color1 = "|cFFFFFFFF"
    local color2 = "|cFF82CAFF"

    -- Output title and display slash commands
    DEFAULT_CHAT_FRAME:AddMessage(color0 .. DKSounds.Title .. color1 ..
    " loaded.")
    DEFAULT_CHAT_FRAME:AddMessage(color1 ..
    "For options you can use " .. color2 .. "/DKSounds" .. color1 ..
    " or " .. color2 .. "/DKS" .. color1 .. ".")
    
end

-- Detect Combat Log Events
function DKSounds.events.COMBAT_LOG_EVENT_UNFILTERED
    (timestamp, event, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)

    -- Debugging events, spell ID's, etc
    if (DKSounds.Debug == 1) then
        DEFAULT_CHAT_FRAME:AddMessage("|----------------------------|")
        if (timestamp ~= nil) then
            DEFAULT_CHAT_FRAME:AddMessage(timestamp)
        end
        if (event ~= nil) then 
            DEFAULT_CHAT_FRAME:AddMessage(event)
        end
        if (srcGUID ~= nil) then 
            DEFAULT_CHAT_FRAME:AddMessage(srcGUID)
        end
        if (srcName ~= nil) then 
            DEFAULT_CHAT_FRAME:AddMessage(srcName)
        end
        if (srcFlags ~= nil) then 
            DEFAULT_CHAT_FRAME:AddMessage(srcFlags)
        end
        if (dstGUID ~= nil) then 
            DEFAULT_CHAT_FRAME:AddMessage(dstGUID)
        end
        if (dstName ~= nil) then 
            DEFAULT_CHAT_FRAME:AddMessage(dstName)
        end
        if (dstFlags ~= nil) then 
            DEFAULT_CHAT_FRAME:AddMessage(dstFlags)
        end
        DEFAULT_CHAT_FRAME:AddMessage("------------------------------")
    end

    -- Must check level for certain abilities, won't
    -- make global 'cos player can ding in the meantime
    local CharLevel = UnitLevel("player");

    -- Check if RS is still up, if not reset counters    
    if (DKSounds.Spells.RSCounter ~= 0) then

        local DKNow = GetTime()
        
        -- Debugger Time Count to check time :)
        -- DEFAULT_CHAT_FRAME:AddMessage(DKNow-DKSounds.Spells.RSCounter)

        -- If 20 secs are passed since it didn't reset or was used,
        -- reset it now 'cos it can't be cast no longer until new proc
        if ((DKNow - DKSounds.Spells.RSCounter) > 20) then
            DKSounds.Spells.RSCounter = 0
            DKSounds.Spells.RSEnabled = false
        end
    end

    if (srcName == DKSounds.CharName) then
        if (event == "SPELL_CAST_SUCCESS") then       
            local sid, spellName = ...

            -- Debugging events, spell ID's, etc
            if (DKSounds.Debug == 1) then
                DEFAULT_CHAT_FRAME:AddMessage("IF_SPELL_CAST_SUCCESS")        
                if (sid ~= nil) then 
                    DEFAULT_CHAT_FRAME:AddMessage(sid)
                end
                if (spellName ~= nil) then 
                    DEFAULT_CHAT_FRAME:AddMessage(spellName)
                end
                DEFAULT_CHAT_FRAME:AddMessage("|----------------------------|")
            end

            -- Death Grip
            if (spellName == DKSounds.Spells.DG) then
    
                -- Since we have 2 sounds we can use
                -- let's randomize :)
                local NumberRandom = math.random(2);   
                 
                if (NumberRandom == 1) then  
                    if (DKSoundsData.config.DG) and (DKSoundsData.config.Snds) then
                        PlaySoundFile("Interface\\AddOns\\DK_Sounds\\Sounds\\DeathGrip1.mp3");
                    end
                    if (DKSoundsData.config.Msgs) then
                        SendChatMessage("Come Here!!!", "SAY");		  
                    end
    
                elseif (NumberRandom == 2) then 
                    if (DKSoundsData.config.DG) and (DKSoundsData.config.Snds) then
                        PlaySoundFile("Interface\\AddOns\\DK_Sounds\\Sounds\\DeathGrip2.mp3");
                    end  
                    if (DKSoundsData.config.Msgs) then
                        SendChatMessage("Get Over Here!!!", "SAY");
                    end
                end 
    
            -- Reset counters for RS since Rune Strike was casted
            elseif (spellName == DKSounds.Spells.RS) then
                if (DKSoundsData.config.RS) then
                    DKSounds.Spells.RSCounter = 0
                    DKSounds.Spells.RSEnabled = false
                end
    
            -- Strangulate
    		elseif (spellName == DKSounds.Spells.Str) then
                if (DKSoundsData.config.Str) and (DKSoundsData.config.Snds) then
                    PlaySoundFile("Interface\\AddOns\\DK_Sounds\\Sounds\\Strangulate.mp3");
                end
                if (DKSoundsData.config.Msgs) then
                    SendChatMessage("I find your lack of faith disturbing", "SAY");
                end
    
            -- Blood Boil    
            elseif (spellName == DKSounds.Spells.BB) then
                if (DKSoundsData.config.BB) and (DKSoundsData.config.Snds) then
                    PlaySoundFile("Interface\\AddOns\\DK_Sounds\\Sounds\\BloodBoil.mp3");
                end
                if (DKSoundsData.config.Msgs) then
                    SendChatMessage("Your pain shall be Legendary!", "SAY");
                end

            -- Blood Strike
            elseif (spellName == DKSounds.Spells.BS) then
                if (DKSoundsData.config.BS) and (DKSoundsData.config.Snds) then
                    PlaySoundFile("Interface\\AddOns\\DK_Sounds\\Sounds\\BloodStrike.mp3");
                end
                if (DKSoundsData.config.Msgs) then
                    SendChatMessage("You will know endless torment!", "SAY");
                end

            -- Scourge Strike / Frost Strike / Heart Strike    
            elseif (spellName == DKSounds.Spells.SS) or
            (spellName == DKSounds.Spells.FS) or (spellName == DKSounds.Spells.HS) then 
                if (DKSoundsData.config.SS) and (DKSoundsData.config.Snds) then
                    PlaySoundFile("Interface\\AddOns\\DK_Sounds\\Sounds\\ScourgeStrike.mp3");
                end
                if (DKSoundsData.config.Msgs) then
                    SendChatMessage("Glory to the Scourge!", "SAY");
                end

            -- Obliterate    
            elseif (spellName == DKSounds.Spells.Obl) then
                if (DKSoundsData.config.Obl) and (DKSoundsData.config.Snds) then
                    PlaySoundFile("Interface\\AddOns\\DK_Sounds\\Sounds\\Obliterate.mp3");
                end
                if (DKSoundsData.config.Msgs) then
                    SendChatMessage("NOW Frostmourne!", "SAY");
                end

            -- Pestilence    
            elseif (spellName == DKSounds.Spells.Pes) then
                if (DKSoundsData.config.Pes) and (DKSoundsData.config.Snds) then
                    PlaySoundFile("Interface\\AddOns\\DK_Sounds\\Sounds\\Pestilence.mp3");
                end
                if (DKSoundsData.config.Msgs) then
                    SendChatMessage("I will make sure, you suffer!!!", "SAY");
                end

            -- Summon Gargoyle
            elseif (spellName == DKSounds.Spells.SG) then
                if (DKSoundsData.config.SG) and (DKSoundsData.config.Snds) then
                    PlaySoundFile("Interface\\AddOns\\DK_Sounds\\Sounds\\Gargoyle.mp3");
                end
                if (DKSoundsData.config.Msgs) then
                    SendChatMessage("Summoning is complete!", "SAY");
                end

--            Moved to SPELL_CAST_START so default sound don't overlap with default sound		    
--            elseif (spellName == DKSounds.Spells.Mount) then

            -- Cannibalize (for Undead DK's)
            elseif (spellName == DKSounds.Spells.Cann) then
                -- Since we have 2 sounds we can use
                -- let's randomize :)
                local NumberRandom = math.random(2);   
                 
                if (NumberRandom == 1) then  
                    if (DKSoundsData.config.Cann) and (DKSoundsData.config.Snds) then
                        PlaySoundFile("Interface\\AddOns\\DK_Sounds\\Sounds\\Cannibalize1.mp3");
                    end
                    if (DKSoundsData.config.Msgs) then
                        SendChatMessage("Me eat dead people!", "SAY");		  
                    end
                elseif (NumberRandom == 2) then 
                    if (DKSoundsData.config.Cann) and (DKSoundsData.config.Snds) then
                        PlaySoundFile("Interface\\AddOns\\DK_Sounds\\Sounds\\Cannibalize2.mp3");
                    end  
                    if (DKSoundsData.config.Msgs) then
                        SendChatMessage("Me eat dead brains!", "SAY");
                    end
                end
            end 
    
        elseif (event == "SPELL_CAST_START") then 
            local sid, spellName = ...

            -- Debugging events, spell ID's, etc
            if (DKSounds.Debug == 1) then
                DEFAULT_CHAT_FRAME:AddMessage("IF_SPELL_CAST_START")        
                if (sid ~= nil) then 
                    DEFAULT_CHAT_FRAME:AddMessage(sid)
                end
                if (spellName ~= nil) then 
                    DEFAULT_CHAT_FRAME:AddMessage(spellName)
                end
                DEFAULT_CHAT_FRAME:AddMessage("|----------------------------|")
            end

            -- Detects both specific class DK Mounts - Acherus Deathcharger and
            -- the flying mount, Winged Steed of the Ebon Blade    
            if (spellName == DKSounds.Spells.AD) or (spellName == DKSounds.Spells.WS) then
                if (DKSoundsData.config.Mount) and (DKSoundsData.config.Snds) then
                    PlaySoundFile("Interface\\AddOns\\DK_Sounds\\Sounds\\Mount.mp3");
                end
                if (DKSoundsData.config.Msgs) then
                    SendChatMessage("I'm the One Horsemen of the Apocalypse!", "SAY");
                end
            end

        -- This type of event has to be here, normally it's catch by srcName
        -- since it's the 1st part of this condition
    	elseif (event == "SPELL_AURA_APPLIED")  and (srcName == DKSounds.CharName) then
            local sid, spellName = ...

            -- Debugging events, spell ID's, etc
            if (DKSounds.Debug == 1) then
                DEFAULT_CHAT_FRAME:AddMessage("IF_SPELL_AURA_APPLIED")        
                if (sid ~= nil) then 
                    DEFAULT_CHAT_FRAME:AddMessage(sid)
                end
                if (spellName ~= nil) then 
                    DEFAULT_CHAT_FRAME:AddMessage(spellName)
                end
                DEFAULT_CHAT_FRAME:AddMessage("|----------------------------|")
            end

            -- Horn of Winter
            if (spellName == DKSounds.Spells.HW) then
                if (DKSoundsData.config.HW) and (DKSoundsData.config.Snds) then
                    PlaySoundFile("Interface\\AddOns\\DK_Sounds\\Sounds\\HornWinter.mp3");
                end
                if (DKSoundsData.config.Msgs) then
                    SendChatMessage("The Lich King has given me true power!", "SAY");
                end
    
            -- Unholy Blight (Unholy Talent)
            elseif (spellName == DKSounds.Spells.UB) then
                if (DKSoundsData.config.UB) and (DKSoundsData.config.Snds) then
                    PlaySoundFile("Interface\\AddOns\\DK_Sounds\\Sounds\\UnholyBlight.mp3");
                end
                if (DKSoundsData.config.Msgs) then
                    SendChatMessage("*Frostmourne chants*", "emote");
                end

            -- Army of the Dead
            elseif (spellName == DKSounds.Spells.AoD) then
                if (DKSoundsData.config.AoD) and (DKSoundsData.config.Snds) then
                    PlaySoundFile("Interface\\AddOns\\DK_Sounds\\Sounds\\ShadowDeath.mp3");
                end
                if (DKSoundsData.config.Msgs) then
                    SendChatMessage("The damned return!", "SAY");
                end
            end

        -- More accurate to grab summons, not all appeared on self,
        -- Raise Dead won't appear on SPELL_CAST, but Army of the Dead
        -- can't be here 'cos each ghoul will trigger this and will spam sound
    	elseif (event == "SPELL_SUMMON") and (srcName == DKSounds.CharName) then
            local sid, spellName = ...            

            -- Debugging events, spell ID's, etc
            if (DKSounds.Debug == 1) then
                DEFAULT_CHAT_FRAME:AddMessage("IF_SPELL_SUMMON")        
                if (sid ~= nil) then 
                    DEFAULT_CHAT_FRAME:AddMessage(sid)
                end
                if (spellName ~= nil) then 
                    DEFAULT_CHAT_FRAME:AddMessage(spellName)
                end
                DEFAULT_CHAT_FRAME:AddMessage("|----------------------------|")
            end
            
            -- Raise Dead
            if (spellName == DKSounds.Spells.RD) then
                if (DKSoundsData.config.RD) and (DKSoundsData.config.Snds) then
                    PlaySoundFile("Interface\\AddOns\\DK_Sounds\\Sounds\\RaiseDead.mp3");
                end
                if (DKSoundsData.config.Msgs) then
                    SendChatMessage("For the Lich King!", "SAY");
                end
            end           
    	end

    -- Other events that normally are fired into dstName, etc
    else
    	if (event == "SPELL_AURA_REFRESH") and (dstName == DKSounds.CharName) then
            local sid, spellName = ...

            -- Debugging events, spell ID's, etc
            if (DKSounds.Debug == 1) then
                DEFAULT_CHAT_FRAME:AddMessage("IF_SPELL_AURA_REFRESH")        
                if (sid ~= nil) then 
                    DEFAULT_CHAT_FRAME:AddMessage(sid)
                end
                if (spellName ~= nil) then 
                    DEFAULT_CHAT_FRAME:AddMessage(spellName)
                end
                DEFAULT_CHAT_FRAME:AddMessage("|----------------------------|")
            end

            -- Horn of Winter
            if (spellName == DKSounds.Spells.HW) then
                if (DKSoundsData.config.HW) and (DKSoundsData.config.Snds) then
                    PlaySoundFile("Interface\\AddOns\\DK_Sounds\\Sounds\\HornWinter.mp3");
                end
                if (DKSoundsData.config.Msgs) then
                    SendChatMessage("The Lich King has given me true power!", "SAY");
                end
            end

        -- Since check is based on not casted abilities must have a level check
        elseif (event == "SWING_MISSED") and (dstName == DKSounds.CharName)
        and (CharLevel > 66) then
    		local MissParsed = ...
    
            -- Since Rune Strike proc don't appear on Combat Log, we must search
            -- for what triggers it, it procs after WE dodged or parried an attack
            -- not the target, as some people may think ;)
    		if ((MissParsed == "PARRY") or (MissParsed == "DODGE")) then
                if (DKSoundsData.config.RS) and (DKSoundsData.config.Snds) then
                    -- Start counter every it procs
                    DKSounds.Spells.RSCounter = GetTime()

--    Debugging proc      
--    DEFAULT_CHAT_FRAME:AddMessage("RUNE STRIKE PROC");

                    -- Avoids playing sound when proc is up, can be rly spammy,
                    -- specially when fighting lower lvls, since this procs
                    -- based on dodge and parry
--                    if (not DKSounds.Spells.Enabled) then
                        PlaySoundFile("Interface\\AddOns\\DK_Sounds\\Sounds\\RuneStrike.mp3");
                        -- Enable flag to avoid spamming afterwards
                        DKSounds.Spells.Enabled = true
    
                        if (DKSoundsData.config.Msgs) then
                            SendChatMessage("Frostmourne Hungers!", "SAY");
                        end
--                    end
                end
            end
        end
    end
end

function DKSounds.events.UNIT_AURA(player)
-- UnitBuff(name, rank, icon, count, debuffType, duration, expirationTime, isMine, isStealable, ...)

--DEFAULT_CHAT_FRAME:AddMessage("start")
--DEFAULT_CHAT_FRAME:AddMessage(DKSounds.Spells.Mounted)

--    DelayCheck = DelayMng.CreateDelayHandler(l_CheckMountedStatus, nil, 0.5, false, false);

    -- Only make sense to check buffs while mounted,
    -- since we're searching for mounting buffs
--    if IsMounted(DKSounds.CharName) and (not DKSounds.Spells.Mounted) then
    if (DKSounds.Spells.Mounted ~= 1) then
        local buffIndex = 1
        local buff = UnitBuff(DKSounds.CharName, buffIndex)
    
        -- Loop through Buffs
        while buff do
--            DEFAULT_CHAT_FRAME:AddMessage(buff)
            -- Check if buff is Mounted
            if (buff == DKSounds.Spells.WS) then
                    -- To avoid more spams of msg and sound
                    DKSounds.Spells.Mounted = 1 
                    if (DKSoundsData.config.Mount) and (DKSoundsData.config.Snds) then
                        PlaySoundFile("Interface\\AddOns\\DK_Sounds\\Sounds\\Mount.mp3");
                    end
                    if (DKSoundsData.config.Msgs) then
                        SendChatMessage("I'm the One Horsemen of the Apocalypse!", "SAY");
                    end
                break
            end
            buffIndex = buffIndex + 1
            buff = UnitBuff(DKSounds.CharName, buffIndex)
        end

    -- Dismounted so reset Mounted value
    elseif (not IsMounted(DKSounds.CharName)) and DKSounds.Spells.Mounted then
        DKSounds.Spells.Mounted = 0 

    end
--DEFAULT_CHAT_FRAME:AddMessage("end")
--DEFAULT_CHAT_FRAME:AddMessage(DKSounds.Spells.Mounted)

end


function DKSounds:CreateOptionFrame()
    local panel = CreateFrame("FRAME", "DKOptions");
    panel.name = "DK Sounds";

    -- Creation of the Labels
    local optString1 = panel:CreateFontString("DKOptions_Str1","OVERLAY","GameFontNormal")
    local optString2 = panel:CreateFontString("DKOptions_Str2","OVERLAY","GameFontNormal")
    local optString3 = panel:CreateFontString("DKOptions_Str3","OVERLAY","GameFontNormal")
    local optString4 = panel:CreateFontString("DKOptions_Str4","OVERLAY","GameFontNormal")
    local optString5 = panel:CreateFontString("DKOptions_Str5","OVERLAY","GameFontNormal")
    local optString6 = panel:CreateFontString("DKOptions_Str6","OVERLAY","GameFontNormal")
    local optString7 = panel:CreateFontString("DKOptions_Str7","OVERLAY","GameFontNormal")
    local optString8 = panel:CreateFontString("DKOptions_Str8","OVERLAY","GameFontNormal")
    local optString9 = panel:CreateFontString("DKOptions_Str9","OVERLAY","GameFontNormal")
    local optString10 = panel:CreateFontString("DKOptions_Str10","OVERLAY","GameFontNormal")
    local optString11 = panel:CreateFontString("DKOptions_Str11","OVERLAY","GameFontNormal")
    local optString12 = panel:CreateFontString("DKOptions_Str12","OVERLAY","GameFontNormal")
    local optString13 = panel:CreateFontString("DKOptions_Str13","OVERLAY","GameFontNormal")
    local optString14 = panel:CreateFontString("DKOptions_Str14","OVERLAY","GameFontNormal")
    local optString15 = panel:CreateFontString("DKOptions_Str15","OVERLAY","GameFontNormal")
    local optString16 = panel:CreateFontString("DKOptions_Str16","OVERLAY","GameFontNormal")
    local optString161 = panel:CreateFontString("DKOptions_Str16","OVERLAY","GameFontNormal")
    local optString17 = panel:CreateFontString("DKOptions_Str16","OVERLAY","GameFontNormal")

    -- Label Descriptions
    optString1:SetText("Enable all Messages:")
    optString2:SetText("Enable all Sounds:")
    optString3:SetText("Death Grip")
    optString4:SetText("Rune Strike")
    optString5:SetText("Strangulate")
    optString6:SetText("Raise Dead")
    optString7:SetText("Horn of Winter")
    optString8:SetText("Blood Boil")
    optString9:SetText("Blood Strike")
    optString10:SetText("Spec Strike")
    optString11:SetText("Obliterate")
    optString12:SetText("Pestilence")
    optString13:SetText("Unholy Blight")
    optString14:SetText("Summon Gargoyle")
    optString15:SetText("DK Mounts (Only)")
    optString16:SetText("Cannibalize")
    optString161:SetText("(for Undead DKs)")
    optString17:SetText("Army of the Dead")
    
    -- Position of Labels
    optString1:SetPoint("TOPLEFT", 10, -20)
    optString2:SetPoint("TOPLEFT", 10, -95)
    optString3:SetPoint("TOPLEFT", 10, -120)
    optString4:SetPoint("TOPLEFT", 10, -140)
    optString5:SetPoint("TOPLEFT", 10, -160)
    optString6:SetPoint("TOPLEFT", 10, -180)
    optString7:SetPoint("TOPLEFT", 10, -200)
    optString8:SetPoint("TOPLEFT", 10, -220)
    optString9:SetPoint("TOPLEFT", 10, -240)
    optString10:SetPoint("TOPLEFT", 220, -120)
    optString11:SetPoint("TOPLEFT", 220, -140)
    optString12:SetPoint("TOPLEFT", 220, -160)
    optString13:SetPoint("TOPLEFT", 220, -180)
    optString14:SetPoint("TOPLEFT", 220, -200)
    optString15:SetPoint("TOPLEFT", 220, -220)
    optString16:SetPoint("TOPLEFT", 220, -240)
    optString161:SetPoint("TOPLEFT", 220, -260)
    optString17:SetPoint("TOPLEFT", 10, -260)

    -- Creation of the Checkboxes
    local checkbox1 = CreateFrame("CheckButton", "$parent_cb1", panel, "OptionsCheckButtonTemplate")
    local checkbox2 = CreateFrame("CheckButton", "$parent_cb2", panel, "OptionsCheckButtonTemplate")
    local checkbox3 = CreateFrame("CheckButton", "$parent_cb3", panel, "OptionsCheckButtonTemplate")
    local checkbox4 = CreateFrame("CheckButton", "$parent_cb4", panel, "OptionsCheckButtonTemplate")
    local checkbox5 = CreateFrame("CheckButton", "$parent_cb5", panel, "OptionsCheckButtonTemplate")
    local checkbox6 = CreateFrame("CheckButton", "$parent_cb6", panel, "OptionsCheckButtonTemplate")
    local checkbox7 = CreateFrame("CheckButton", "$parent_cb7", panel, "OptionsCheckButtonTemplate")
    local checkbox8 = CreateFrame("CheckButton", "$parent_cb8", panel, "OptionsCheckButtonTemplate")
    local checkbox9 = CreateFrame("CheckButton", "$parent_cb9", panel, "OptionsCheckButtonTemplate")
    local checkbox10 = CreateFrame("CheckButton", "$parent_cb10", panel, "OptionsCheckButtonTemplate")
    local checkbox11 = CreateFrame("CheckButton", "$parent_cb11", panel, "OptionsCheckButtonTemplate")
    local checkbox12 = CreateFrame("CheckButton", "$parent_cb12", panel, "OptionsCheckButtonTemplate")
    local checkbox13 = CreateFrame("CheckButton", "$parent_cb13", panel, "OptionsCheckButtonTemplate")
    local checkbox14 = CreateFrame("CheckButton", "$parent_cb14", panel, "OptionsCheckButtonTemplate")
    local checkbox15 = CreateFrame("CheckButton", "$parent_cb15", panel, "OptionsCheckButtonTemplate")
    local checkbox16 = CreateFrame("CheckButton", "$parent_cb16", panel, "OptionsCheckButtonTemplate")
    local checkbox17 = CreateFrame("CheckButton", "$parent_cb17", panel, "OptionsCheckButtonTemplate")
    
    -- Size of the Checkboxes
    checkbox1:SetWidth(18); checkbox1:SetHeight(18)
    checkbox2:SetWidth(18); checkbox2:SetHeight(18)
    checkbox3:SetWidth(18); checkbox3:SetHeight(18)
    checkbox4:SetWidth(18); checkbox4:SetHeight(18)
    checkbox5:SetWidth(18); checkbox5:SetHeight(18)
    checkbox6:SetWidth(18); checkbox6:SetHeight(18)
    checkbox7:SetWidth(18); checkbox7:SetHeight(18)
    checkbox8:SetWidth(18); checkbox8:SetHeight(18)
    checkbox9:SetWidth(18); checkbox9:SetHeight(18)
    checkbox10:SetWidth(18); checkbox10:SetHeight(18)
    checkbox11:SetWidth(18); checkbox11:SetHeight(18)
    checkbox12:SetWidth(18); checkbox12:SetHeight(18)
    checkbox13:SetWidth(18); checkbox13:SetHeight(18)
    checkbox14:SetWidth(18); checkbox14:SetHeight(18)
    checkbox15:SetWidth(18); checkbox15:SetHeight(18)
    checkbox16:SetWidth(18); checkbox16:SetHeight(18)
    checkbox17:SetWidth(18); checkbox17:SetHeight(18)

    -- Positioning of the Checkboxes
    checkbox1:SetPoint("TOPRIGHT", -230, -20)
    checkbox2:SetPoint("TOPRIGHT", -230, -95)
    checkbox3:SetPoint("TOPRIGHT", -230, -120)
    checkbox4:SetPoint("TOPRIGHT", -230, -140)
    checkbox5:SetPoint("TOPRIGHT", -230, -160)
    checkbox6:SetPoint("TOPRIGHT", -230, -180)
    checkbox7:SetPoint("TOPRIGHT", -230, -200)
    checkbox8:SetPoint("TOPRIGHT", -230, -220)
    checkbox9:SetPoint("TOPRIGHT", -230, -240)
    checkbox10:SetPoint("TOPRIGHT", -20, -120)
    checkbox11:SetPoint("TOPRIGHT", -20, -140)
    checkbox12:SetPoint("TOPRIGHT", -20, -160)
    checkbox13:SetPoint("TOPRIGHT", -20, -180)
    checkbox14:SetPoint("TOPRIGHT", -20, -200)
    checkbox15:SetPoint("TOPRIGHT", -20, -220)
    checkbox16:SetPoint("TOPRIGHT", -20, -240)
    checkbox17:SetPoint("TOPRIGHT", -230, -260)
    
    -- Check initial status of Checkboxes
    checkbox1:SetChecked(DKSoundsData.config.Msgs)
    checkbox2:SetChecked(DKSoundsData.config.Snds)
    checkbox3:SetChecked(DKSoundsData.config.DG)
    checkbox4:SetChecked(DKSoundsData.config.RS)
    checkbox5:SetChecked(DKSoundsData.config.Str)
    checkbox6:SetChecked(DKSoundsData.config.RD)
    checkbox7:SetChecked(DKSoundsData.config.HW)
    checkbox8:SetChecked(DKSoundsData.config.BB)
    checkbox9:SetChecked(DKSoundsData.config.BS)
    checkbox10:SetChecked(DKSoundsData.config.SS)
    checkbox11:SetChecked(DKSoundsData.config.Obl)
    checkbox12:SetChecked(DKSoundsData.config.Pes)
    checkbox13:SetChecked(DKSoundsData.config.UB)
    checkbox14:SetChecked(DKSoundsData.config.SG)
    checkbox15:SetChecked(DKSoundsData.config.Mount)
    checkbox16:SetChecked(DKSoundsData.config.Cann)
    checkbox17:SetChecked(DKSoundsData.config.AoD)
    
    -- Definition of the event OnClick()    
    checkbox1:SetScript("OnClick", function() DKSounds:ToggleMsgs() end)
    checkbox2:SetScript("OnClick",
        function()
            DKSounds:ToggleSnds()
            -- Based if Enable / Disable all sounds options
            -- the rest of the checkboxes may have to be
            -- disabled or enabled based on checkbox2
            if (DKSoundsData.config.Snds) then 
                checkbox3:Enable()
                checkbox4:Enable()
                checkbox5:Enable()
                checkbox6:Enable()
                checkbox7:Enable()
                checkbox8:Enable()
                checkbox9:Enable()
                checkbox10:Enable()
                checkbox11:Enable()
                checkbox12:Enable()
                checkbox13:Enable()
                checkbox14:Enable()
                checkbox15:Enable()
                checkbox16:Enable()
                checkbox17:Enable()
            else
                checkbox3:Disable()
                checkbox4:Disable()
                checkbox5:Disable()
                checkbox6:Disable()
                checkbox7:Disable()
                checkbox8:Disable()
                checkbox9:Disable()
                checkbox10:Disable()
                checkbox11:Disable()
                checkbox12:Disable()
                checkbox13:Disable()
                checkbox14:Disable()
                checkbox15:Disable()
                checkbox16:Disable()
                checkbox17:Disable()
            end
        end)
    checkbox3:SetScript("OnClick", function() DKSounds:ToggleDG() end)
    checkbox4:SetScript("OnClick", function() DKSounds:ToggleRS() end)
    checkbox5:SetScript("OnClick", function() DKSounds:ToggleStr() end)
    checkbox6:SetScript("OnClick", function() DKSounds:ToggleRD() end)
    checkbox7:SetScript("OnClick", function() DKSounds:ToggleHW() end)
    checkbox8:SetScript("OnClick", function() DKSounds:ToggleBB() end)
    checkbox9:SetScript("OnClick", function() DKSounds:ToggleBS() end)
    checkbox10:SetScript("OnClick", function() DKSounds:ToggleSS() end)
    checkbox11:SetScript("OnClick", function() DKSounds:ToggleObl() end)
    checkbox12:SetScript("OnClick", function() DKSounds:TogglePes() end)
    checkbox13:SetScript("OnClick", function() DKSounds:ToggleUB() end)
    checkbox14:SetScript("OnClick", function() DKSounds:ToggleSG() end)
    checkbox15:SetScript("OnClick", function() DKSounds:ToggleMount() end)
    checkbox16:SetScript("OnClick", function() DKSounds:ToggleCann() end)
    checkbox17:SetScript("OnClick", function() DKSounds:ToggleAoD() end)

    -- Author / Creditss
    local authString1 = panel:CreateFontString("DKAuth_Str1","OVERLAY","GameFontNormal")
    local authString2 = panel:CreateFontString("DKAuth_Str2","OVERLAY","GameFontNormal")
    
    -- Label Descriptions
    authString1:SetText(DKSounds.Title .. " - Created by NTex")
    authString2:SetText("AKA Hazïel from EU-Bladefist")
    
    -- Position of Labels
    authString1:SetPoint("TOPRIGHT", -20, -385)
    authString2:SetPoint("TOPRIGHT", -20, -400)
    
    InterfaceOptions_AddCategory(panel); 
end

function DKSounds.Options()
    InterfaceOptionsFrame_OpenToCategory(getglobal("DKOptions"))
end

-- Input / Output / Toggle Options
function DKSounds:ToggleMsgs()
    DKSoundsData.config.Msgs = not DKSoundsData.config.Msgs
end

function DKSounds:ToggleSnds()
    DKSoundsData.config.Snds = not DKSoundsData.config.Snds
end

function DKSounds:ToggleDG()
    DKSoundsData.config.DG = not DKSoundsData.config.DG
end

function DKSounds:ToggleRS()
    DKSoundsData.config.RS = not DKSoundsData.config.RS
end

function DKSounds:ToggleStr()
    DKSoundsData.config.Str = not DKSoundsData.config.Str
end

function DKSounds:ToggleRD()
    DKSoundsData.config.RD = not DKSoundsData.config.RD
end

function DKSounds:ToggleHW()
    DKSoundsData.config.HW = not DKSoundsData.config.HW
end

function DKSounds:ToggleBB()
    DKSoundsData.config.BB = not DKSoundsData.config.BB
end

function DKSounds:ToggleBS()
    DKSoundsData.config.BS = not DKSoundsData.config.BS
end

function DKSounds:ToggleSS()
    DKSoundsData.config.SS = not DKSoundsData.config.SS
end

function DKSounds:ToggleObl()
    DKSoundsData.config.Obl = not DKSoundsData.config.Obl
end

function DKSounds:TogglePes()
    DKSoundsData.config.Pes = not DKSoundsData.config.Pes
end

function DKSounds:ToggleUB()
    DKSoundsData.config.UB = not DKSoundsData.config.UB
end

function DKSounds:ToggleSG()
    DKSoundsData.config.SG = not DKSoundsData.config.SG
end

function DKSounds:ToggleMount()
    DKSoundsData.config.Mount = not DKSoundsData.config.Mount
end

function DKSounds:ToggleCann()
    DKSoundsData.config.Cann = not DKSoundsData.config.Cann
end

function DKSounds:ToggleAoD()
    DKSoundsData.config.AoD = not DKSoundsData.config.AoD
end
-- Input / Output / Toggle Options
